import logging

from django.core.urlresolvers import resolve, reverse

from django_auth_policy.forms import StrictPasswordChangeForm
from django_auth_policy.utils import update_session
from django_auth_policy.settings import (CHANGE_PASSWORD_VIEW_NAME,
    LOGIN_VIEW_NAME, LOGOUT_VIEW_NAME)


logger = logging.getLogger(__name__)


class AuthenticationPolicyMiddleware(object):
    """ This middleware enforces the following policy:
    - Change of password when password has expired;
    - Change of password when user has a temporary password;
    - Logout disabled users;

    This is enforced using middleware to prevent users from accessing any page
    handled by Django without the policy being enforced.
    """
    change_password_path = reverse(CHANGE_PASSWORD_VIEW_NAME)
    login_path = reverse(LOGIN_VIEW_NAME)
    logout_path = reverse(LOGOUT_VIEW_NAME)

    def process_request(self, request):
        assert hasattr(request, 'user'), (
            'AuthenticationPolicyMiddleware needs a user attribute on '
            'request, add AuthenticationMiddleware before '
            'AuthenticationPolicyMiddleware in MIDDLEWARE_CLASSES')

        if not request.user.is_authenticated():
            return None

        # Log out disabled users
        if not request.user.is_active:
            logger.warning('Log out inactive user, user=%s', request.user)
            view_func, args, kwargs = resolve(self.logout_path)
            return view_func(request, *args, **kwargs)

        # Do not do password change for certain URLs
        if request.path in (self.change_password_path, self.login_path,
                            self.logout_path):
            return None

        # Check for 'enforce_password_change' in session set by login view
        if request.session.get('password_change_enforce', False):
            return self.password_change(request)

        return None

    def process_response(self, request, response):
        if not hasattr(request, 'user') or not request.user.is_authenticated():
            return response

        # When password change is enforced, check if this is still required
        # for next request
        if not request.session.get('password_change_enforce', False):
            return response

        update_session(request.session, request.user)

        return response

    def password_change(self, request):
        """ Return 'password_change' view.
        This resolves the view with the name 'password_change'.

        Overwrite this method when needed.
        """
        view_func, args, kwargs = resolve(self.change_password_path)

        assert issubclass(kwargs['password_change_form'],
                          StrictPasswordChangeForm), (
            "Use django_auth_policy StrictPasswordChangeForm for password "
            "changes.")

        # Provide extra context to be used in the password_change template
        is_exp = request.session.get('password_is_expired', False)
        is_tmp = request.session.get('password_is_temporary', False)
        if not 'extra_context' in kwargs:
            kwargs['extra_context'] = {}
        kwargs['extra_context']['is_enforced'] = True
        kwargs['extra_context']['is_temporary'] = is_tmp
        kwargs['extra_context']['is_expired'] = is_exp
        return view_func(request, *args, **kwargs)



import logging

from django.utils.translation import ugettext as _

from django_auth_policy.checks import enforce_password_change
from django_auth_policy import settings as dap_settings


logger = logging.getLogger(__name__)


def update_session(session, user):
    """ Check for temporary or expired passwords and store in session
    Our middleware should enforce a password change in next request
    """
    enforce, is_exp, is_temp = enforce_password_change(user)

    # Log password enforcement
    if enforce:
        if is_temp and not session.get('password_is_temporary'):
            logger.info(u'User %s must change temporary password', user)

        if is_exp and not session.get('password_is_expired'):
            logger.info(u'User %s must change expired password', user)

        if (not is_temp and not is_exp and
            not session.get('password_change_enforce')):
            logger.info(u'User %s must change password', user)

    session['password_change_enforce'] = enforce
    session['password_is_expired'] = is_exp
    session['password_is_temporary'] = is_temp


def password_policy_texts():
    """ Returns a list of policy rules for passwords as enforces by
    Django Auth Policy. Use this to display policy rules on pages
    with change password forms.

    Every rule has a text and an optional caption.
    """
    rules = []

    if dap_settings.PASSWORD_MIN_LENGTH:
        rules.append({
            'text': _(dap_settings.PASSWORD_MIN_LENGTH_TEXT).format(
                length=dap_settings.PASSWORD_MIN_LENGTH),
            'caption': '',
        })

    for rule in dap_settings.PASSWORD_COMPLEXITY:
        rules.append({
            'text': _(dap_settings.PASSWORD_COMPLEXITY_TEXT).format(
                rule_text=_(rule['text'])),
            'caption': _('Allowed characters: {chars}').format(
                chars=rule['chars']),
        })

    if dap_settings.MAX_PASSWORD_AGE:
        rules.append({
            'text': _(dap_settings.MAX_PASSWORD_AGE_TEXT).format(
                age=dap_settings.MAX_PASSWORD_AGE),
            'caption': '',
        })

    return rules



from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

from django_auth_policy import settings as dap_settings


def password_min_length(value):
    if dap_settings.PASSWORD_MIN_LENGTH_TEXT is None:
        return

    if len(value) < dap_settings.PASSWORD_MIN_LENGTH:
        msg = _(dap_settings.PASSWORD_MIN_LENGTH_TEXT).format(
            length=dap_settings.PASSWORD_MIN_LENGTH)
        raise ValidationError(msg, code='password_min_length')


def password_complexity(value):
    if not dap_settings.PASSWORD_COMPLEXITY:
        return

    pw_set = set(value)
    for rule in dap_settings.PASSWORD_COMPLEXITY:
        if not pw_set.intersection(rule['chars']):
            msg = _(dap_settings.PASSWORD_COMPLEXITY_TEXT).format(
                rule_text=_(rule['text']))
            raise ValidationError(msg, 'password_complexity')
